@@ -1,30 +1,29 @@
 package Chess::PGN::EPD;
 
-use 5.006;
+use feature ':5.10';
 use strict;
 use warnings;
 use Chess::PGN::Moves;
-use DB_File;
+use Storable              qw( retrieve );
 use Cwd                   qw( realpath );
 use File::Spec::Functions qw( catdir );
-use Data::Dumper;
+use Try::Tiny             qw( try catch );
 
 require Exporter;
 
-my ( %hECO, %hNIC, %hOpening );
+my ( $hECO, $hNIC, $hOpening );
 my %hash = (
-    ECO     => \%hECO,
-    NIC     => \%hNIC,
-    Opening => \%hOpening,
+    ECO     => \$hECO,
+    NIC     => \$hNIC,
+    Opening => \$hOpening
 );
 
-
 my $mod = __PACKAGE__;
 $mod =~ s{::}{/}g;
 $mod .= '.pm';
 $mod = $INC{$mod};
-
 $mod =~ s/EPD\.pm//i;
+
 my $module_dir_qfn = realpath($mod);
 my $db_dir_qfn     = catdir($module_dir_qfn, 'db');
 
@@ -32,34 +31,42 @@ unless (-d $db_dir_qfn) {
     $db_dir_qfn = realpath('.\db');
 }
 
-my $ECO_path     = catdir($db_dir_qfn,'ECO');
-my $NIC_path     = catdir($db_dir_qfn,'NIC');
-my $Opening_path = catdir($db_dir_qfn,'Opening');
+my $ECO_path     = catdir($db_dir_qfn,'ECO.stor');
+my $NIC_path     = catdir($db_dir_qfn,'NIC.stor');
+my $Opening_path = catdir($db_dir_qfn,'Opening.stor');
 
-tie %hECO, "DB_File", $ECO_path, O_RDONLY, 0664, $DB_HASH
-  or die "Couldn't tie '$ECO_path': $!\n";
-tie %hNIC, "DB_File", $NIC_path, O_RDONLY, 0664, $DB_HASH
-  or die "Couldn't tie '$NIC_path': $!\n";
-tie %hOpening, "DB_File", $Opening_path, O_RDONLY, 0664, $DB_HASH
-  or die "Couldn't tie '$Opening_path': $!\n";
-
-END {
-    untie %hECO;
-    untie %hNIC;
-    untie %hOpening;
-}
+try {
+    $hECO = retrieve($ECO_path);
+} catch {
+    print "Couldn't open $ECO_path : $!";
+    exit;
+};
+try {
+    $hNIC = retrieve($NIC_path);
+} catch {
+    print "Couldn't open $NIC_path : $!";
+    exit;
+};
+try {
+    $hOpening = retrieve($Opening_path);
+} catch {
+    print "Couldn't open $Opening_path : $!";
+    exit;
+};
 
 our @ISA    = qw(Exporter);
 our @EXPORT = qw(
   &epdcode
   &epdset
+  &epdfromto
   &epdstr
   &epdlist
   &epdgetboard
+  &epdTaxonomy
   &psquares
   %font2map
 );
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 our %font2map = (
     'Chess Cases'           => 'leschemelle',
@@ -91,12 +98,21 @@ our %font2map = (
     'Traveller Standard V3' => 'cowderoy',
 );
 
-my %board;
-my $Kc;
-my $Qc;
-my $kc;
-my $qc;
-my $w;
+my %board = qw(
+  a1 R a2 P a7 p a8 r
+  b1 N b2 P b7 p b8 n
+  c1 B c2 P c7 p c8 b
+  d1 Q d2 P d7 p d8 q
+  e1 K e2 P e7 p e8 k
+  f1 B f2 P f7 p f8 b
+  g1 N g2 P g7 p g8 n
+  h1 R h2 P h7 p h8 r
+);
+my $Kc = 1;
+my $Qc = 1;
+my $kc = 1;
+my $qc = 1;
+my $w  = 1;
 
 my @onwhite = (
     1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0,
@@ -307,14 +323,13 @@ my %convertPalView = (
 );
 
 sub epdcode {
-    my $file = shift;
+    my $key = shift;
     my $epd  = shift;
     my $code;
-    my $h = $hash{$file} or die "Unknown option '$file': $!\n";
+    my $h = ${$hash{$key}};
 
-    for (@$epd) {
-#        $code = %{$h}->{$_};
-        $code = $h->{$_};
+    for ( @{$epd} ) {
+        $code = $h->{$_};           ## no critic
         last if $code;
     }
     return ( $code or 'Unknown' );
@@ -373,6 +388,7 @@ sub epdset {
         $kc = 1;
         $qc = 1;
     }
+    return;
 }
 
 sub epdstr {
@@ -386,8 +402,20 @@ sub epdstr {
         }
         $parameters{'epd'} = epd( 0, 0, 0, 0, 0, 0, %board );
     }
-    my $epd  = $parameters{'epd'}        or die "Missing epd parameter: $!\n";
-    my $type = lc( $parameters{'type'} ) or die "Missing type parameter: $!\n";
+    my $epd;
+    my $type;
+    try {
+        $epd = $parameters{'epd'};
+    } catch {
+        print "Missing epd parameter: $!\n";
+        exit;
+    };
+    try {
+        $type = lc( $parameters{'type'} );
+    } catch {
+        print "Missing type parameter: $!\n";
+        exit;
+    };
     my ( $border, $corner, $legend ) = ( 'single', 'square', 'no' );
 
     $border = lc( $parameters{'border'} ) if exists( $parameters{'border'} );
@@ -521,12 +549,173 @@ sub mappiece {
 }
 
 sub epdgetboard {
-    my $epd = shift;
-
-    epdset($epd);
+    if ( my $epd = shift ) {
+        epdset($epd);
+    }
     return $w, $Kc, $Qc, $kc, $qc, %board;
 }
 
+sub epdfromto {
+    my @moves = @_;
+    my @movelist;
+
+    epdset();
+    for (@moves) {
+        if ($_) {
+            my ( $piece, $to, $from, $promotion ) = movetype( $w, $_ );
+            my $enpassant;
+            my $ep = '-';
+            my $castles = /O/ ? $_ : '';
+
+            if ( $piece eq "P" ) {
+                $piece = "p" if not $w;
+                $promotion = lc($promotion) if $promotion and not $w;
+                if ($from) {
+                    $from .= substr( $to, 1, 1 );
+                    if ($w) {
+                        substr( $from, 1, 1 ) -= 1;
+                    }
+                    else {
+                        $from++;
+                    }
+                }
+                else {
+                    $from = $to;
+
+                    if ($w) {
+                        substr( $from, 1, 1 ) -= 1;
+                        unless ($board{$from}) {
+                            $ep = $from;
+                            substr( $from, 1, 1 ) -= 1;
+                        }
+                    }
+                    else {
+                        $from++;
+                        unless ($board{$from}) {
+                            $ep = $from;
+                            $from++;
+                        }
+                    }
+                }
+
+                if ( substr( $from, 0, 1 ) ne substr( $to, 0, 1 ) ) {
+                    if ( not $board{$to} ) {
+                        $enpassant = $to;
+                        if ($w) {
+                            substr( $enpassant, 1, 1 ) =
+                              chr( ord( substr( $enpassant, 1, 1 ) ) - 1 );
+                        }
+                        else {
+                            substr( $enpassant, 1, 1 ) =
+                              chr( ord( substr( $enpassant, 1, 1 ) ) + 1 );
+                        }
+                        $board{$enpassant} = undef;
+                        $enpassant = $enpassant // '';
+                        $from = $from // '';
+                        $to = $to // '';
+                    }
+                }
+                ( $board{$to}, $board{$from} ) =
+                  ( $promotion ? $promotion : $board{$from}, undef );
+                $piece = $piece // '';
+                $from = $from // '';
+                $to = $to // '';
+                $promotion = $promotion // '';
+            }
+            elsif ( $piece eq "KR" ) {
+                my ( $k_from, $r_from ) = unpack( "A2A2", $from );
+                my ( $k_to,   $r_to )   = unpack( "A2A2", $to );
+
+                ( $board{$k_to}, $board{$k_from} ) = ( $board{$k_from}, undef );
+                ( $board{$r_to}, $board{$r_from} ) = ( $board{$r_from}, undef );
+                if ($w) {
+                    $Kc = $Qc = 0;
+                }
+                else {
+                    $kc = $qc = 0;
+                }
+            }
+            else {
+                my @piece_at;
+                my @fromlist;
+
+                $piece = lc($piece) if not $w;
+                @piece_at = psquares( $piece, %board );
+                if ($from) {
+                    my @tmp;
+
+                    $from = $from // '';
+                    if ( $from =~ /[a-h]/ ) {
+                        for (@piece_at) {
+                            push( @tmp, $_ )
+                              if ( substr( $_, 0, 1 ) eq $from );
+                        }
+                    }
+                    else {
+
+                        for (@piece_at) {
+                            push( @tmp, $_ )
+                              if ( substr( $_, 1, 1 ) eq $from );
+                        }
+                    }
+                    @piece_at = @tmp;
+                }
+
+                for my $square (@piece_at) {
+                    for ( @{ $move_table{ uc($piece) }{$square} } ) {
+                        push( @fromlist, $square ) if $_ eq $to;
+                    }
+                }
+                if ( scalar(@fromlist) != 1 ) {
+                    for (@fromlist) {
+                        if (    canmove( $piece, $to, $_, %board )
+                            and isLegal( $w, $_, $to, %board ) )
+                        {
+                            $from = $_;
+                            last;
+                        }
+                    }
+                }
+                else {
+                    $from = $fromlist[0];
+                }
+                if ( $piece =~ /[RrKk]/ ) {
+                    if ( $piece eq 'R' ) {
+                        $Kc = 0 if $from eq 'h1';
+                        $Qc = 0 if $from eq 'a1';
+                    }
+                    elsif ( $piece eq 'r' ) {
+                        $kc = 0 if $from eq 'h8';
+                        $qc = 0 if $from eq 'a8';
+                    }
+                    elsif ( $piece eq 'K' ) {
+                        $Kc = $Qc = 0;
+                    }
+                    else {
+                        $kc = $qc = 0;
+                    }
+                }
+                ( $board{$to}, $board{$from} ) = ( $board{$from}, undef );
+                $piece = $piece // '';
+                $from = $from // '';
+                $to = $to // '';
+            }
+            my $movehash = {
+                    piece => $piece // '',
+                    from => $from // '',
+                    to => $to // '',
+                    promotion => $promotion // '',
+                    enpassant => $enpassant // '',
+                    castles => $castles // '',
+            };
+            push(@movelist,$movehash);
+            $w ^= 1;
+        }
+    }
+    %board = ();
+    return @movelist;
+}
+
 sub epdlist {
     my @moves = @_;
     my $debug = ( $moves[-1] eq '1' );
@@ -578,12 +767,17 @@ sub epdlist {
 
                     if ($w) {
                         substr( $from, 1, 1 ) -= 1;
-                        $ep = $from, substr( $from, 1, 1 ) -= 1
-                          unless $board{$from};
+                        unless ($board{$from}) {
+                            $ep = $from;
+                            substr( $from, 1, 1 ) -= 1;
+                        }
                     }
                     else {
                         $from++;
-                        $ep = $from, $from++ unless $board{$from};
+                        unless ($board{$from}) {
+                            $ep = $from;
+                            $from++;
+                        }
                     }
                 }
 
@@ -727,7 +921,12 @@ sub epdlist {
                 if ( not $from ) {
                     ShowPieces(%board);
                     Print(%board);
-                    die "From undefined\n" if not $from;
+                    try {
+                        $from;
+                    } catch {
+                        print "From undefined\n";
+                        exit;
+                    };
                 }
             }
             $w ^= 1;
@@ -775,6 +974,7 @@ sub ShowPieces {
         next unless $piece;
         print "'$square' == ", $piece, "\n";
     }
+    return;
 }
 
 sub Print {
@@ -804,6 +1004,7 @@ sub Print {
         $whitesquare ^= 1;
     }
     print "\n   abcdefgh\n\n";
+    return;
 }
 
 sub movetype {
@@ -871,7 +1072,7 @@ sub movetype {
 sub psquares {
     my ( $piece, %board ) = @_;
 
-    grep { $_ and $board{$_} and ( $board{$_} eq $piece ) } keys %board;
+    return grep { $_ and $board{$_} and ( $board{$_} eq $piece ) } keys %board;
 }
 
 sub epd {
@@ -995,6 +1196,38 @@ sub canmove {
     return $result;
 }
 
+sub epdTaxonomy {
+    my (%options) = @_;
+    my @moves = @{$options{'moves'}};
+    my @results;
+    my ($eco,$nic,$opening);
+    my @epd = reverse (epdlist(@moves));
+
+    if ($options{'all'}) {
+        $eco = epdcode('ECO',\@epd);
+        $nic = epdcode('NIC',\@epd);
+        $opening = epdcode('Opening',\@epd);
+    }
+    else {
+        given (lc (keys %options)) {
+            when ('eco') {$eco = epdcode('ECO',\@epd)}
+            when ('nic') {$nic = epdcode('NIC',\@epd)}
+            when ('opening') {$opening = epdcode('Opening',\@epd)}
+        }
+    }
+    if ($options{'astags'}) {
+        push(@results,"[ECO \"$eco\"]") if $eco;
+        push(@results,"[NIC \"$nic\"]") if $nic;
+        push(@results,"[Opening \"$opening\"]") if $opening;
+    }
+    else {
+        push(@results,$eco) if $eco;
+        push(@results,$nic) if $nic;
+        push(@results,$opening) if $opening;
+    }
+    return @results;
+}
+
 1;
 __END__
 
@@ -1004,6 +1237,10 @@ Chess::PGN::EPD - Perl extension to produce and manipulate EPD text.
 
 =head1 SYNOPSIS
 
+=head1 CODE examples
+
+B<THIS>
+
  #!/usr/bin/perl
  #
  #
@@ -1048,10 +1285,8 @@ B<OR>
          my @epd;
  
          $pgn->parse_game();
-         @epd = reverse epdlist( @{$pgn->moves()} );
-         print '[ECO,"',epdcode('ECO',\@epd),"\"]\n";
-         print '[NIC,"',epdcode('NIC',\@epd),"\"]\n";
-         print '[Opening,"',epdcode('Opening',\@epd),"\"]\n";
+         my @moves = @{ $game{'GameMoves'} };
+         print join("\n",epdTaxonomy(moves => \@moves,all => 1,astags => 1)),"\n";
      }
  }
 
@@ -1072,279 +1307,38 @@ Allowed codes are:
 
 =back
 
-At the moment, this routine depends on three Berekely DB files installed along with
+At the moment, this routine depends on three Storable files installed along with
 the module. On demand other database formats may be implemented. The 'ToDo' list
 also mentions the possibility of extending the databases, although that might come in
-the form of a 'How To' rather than any code solution.
+the form of a 'How To' rather than any code solution. NOTE: This routine is deprecated
+in favor of epdTaxonomy and will at some point be removed.
 
-=head2 epdset(I<epd>)
+=head2 epdfromto(I<movelist>)
 
-For those instances where the game in question does not begin
-with a complete move list, this function allows the user to
-set the starting position using a 'EPD' string as described
-elsewhere in the document.
+Returns an array of hashes that contain move information. Useful for conversions to other
+forms of notation; i.e. telegraphic or english notation.
 
-=head2 epdstr(I<epd>|I<board>,I<type> [I<border>,I<corner>,I<legend>])
+=head2 epdgetboard([I<epd>])
 
-Returns an array of strings that represent a diagramatic conversion of the
-specified B<epd> string or board postion to the specified B<type>. Parameters are passed as
-a anonymous hash, i.e. epdstr(epd => $position,type => 'diagram') or similar.
-
-=head3 Types Supported
-
-The following types are understood by B<epdstr>:
+Provides access to the 'board' with the current epd postition set up. If given an EPD string,
+set board accordingly. The returned values are:
 
 =over
 
-=item 'diagram'
+=item $w - boolean, white to move?
 
-A plain ASCII diagram with simple border showing rank and file. Typical output:
+=item $Kc - boolean, has white castled king side?
 
- 8  rnbqkb r
- 7  ppp pppp
- 6   - - n -
- 5  - -P- - 
- 4   - - - -
- 3  - - - - 
- 2  PPPP PPP
- 1  RNBQKBNR
-    abcdefgh
+=item $Qc - boolean, has white castled queen side?
 
-=item 'text'
+=item $kc - boolean, has black castled king side?
 
-A plain ASCII diagram. Typical output:
+=item $qc - boolean, has black castled queen side?
 
- rnbqkb r
- ppp pppp
-  - - n -
- - -P- - 
-  - - - -
- - - - - 
- PPPP PPP
- RNBQKBNR
-
-=item 'palview'
-
-An array of HTML information that represents the tabular diagram information for PalView.
-Typical output:
-
-<IMG SRC="jpc/br.gif"><IMG SRC="jpc/bn.gif"><IMG SRC="jpc/bb.gif"><IMG SRC="jpc/bq.gif"><IMG SRC="jpc/bk.gif"><IMG SRC="jpc/bb.gif"><IMG SRC="jpc/bn.gif"><IMG SRC="jpc/br.gif"><BR>
-<IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><BR>
-<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
-<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
-<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
-<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/wn.gif"><BR>
-<IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><BR>
-<IMG SRC="jpc/wr.gif"><IMG SRC="jpc/wn.gif"><IMG SRC="jpc/wb.gif"><IMG SRC="jpc/wq.gif"><IMG SRC="jpc/wk.gif"><IMG SRC="jpc/wb.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/wr.gif"><BR>
-
-=item 'latex'
-
-The necessary text fragment to 'set' the diagram in LaTeX using 
-any variation of Piet Tutelars original chess12.tar.Z package. As given, the LaTeX
-command 'diagram' is used. As an example here is the source to test.tex:
-
- %%
- %% test.tex -- example LaTeX file to demonstrate output from Chess::PGN::EPD
- %%
- \documentclass{article}
- \usepackage{chess}
- \usepackage{bdfchess}
- \begin{document}
- \newenvironment{diagram}{\begin{nochess}}{$$\showboardwithnotation$$\end{nochess}}
- %%
- %% fragment as produced by epdstr(epd => $position,type => 'latex');
- %%
- \begin{diagram}
- \board
- {rnbqkb r}
- {ppp pppp}
- { * * n *}
- {* *P* * }
- { * * * *}
- {* * * * }
- {PPPP PPP}
- {RNBQKBNR}
- \end{diagram}
- %%
- %% end of fragment
- %%
- \end{document}
-
-=item 'linares'
-
-Alpine Electronics' LinaresDiagram font. Mapping also works with both HastingsDiagram
-and ZurichDiagram fonts. Single or double border, With or without algebraic legend.
-
-=item 'linares1'
-
-Standard mapping, single border, squares offset.
-
-=item 'linares2'
-
-Standard mapping, thick single border.
-
-=item 'tilburg'
-
-A borderless font designed by Eric Schiller and Bill Cone.
-
-=item 'marroquin'
-
-This type refers to any font designed by Armando H. Marroquin,
-excepting his FigurineSymbol fonts. They having a different purpose,
-have a different mapping.
-
-=item 'leschemelle'
-
-The map for Chess Cases designed by Matthieu Leschemelle.
-
-=item 'bentzen1'
-
-The map for Chess Alpha designed by Eric Bentzen.
-
-=item 'bentzen2'
-
-The map for Chess Berlin designed by Eric Bentzen.
-
-=item 'hickey'
-
-The map for Chess Plain designed by Alan Hickey.
-
-=item 'scott1'
-
-The map for Chess Regular a port of Adobe Cheq ported to
-True Type by Alistair Scott.
-
-=item 'scott2'
-
-The map for Chess Usual a modification of Chess Regular
-by Armando H. Marroquin.
-
-=item 'bodlaender'
-
-The map for Chess Utrecht designed by Hans Bodlaender.
-
-=item 'cowderoy'
-
-The map for Traveller Standard v3  designed by Alan Cowderoy.
+=item %board - hash board, keys are algebraic square names, values are occupying piece.
 
 =back
 
-Note that 'type' is not case sensative so that 'latex' and 'LaTeX' will both
-work equally well.
-
-=head3 Fonts Supported
-
-List with font name, font author, and type name:
-
-=over
-
-=item 1Chess Cases -- Matthieu Leschemelle -- leschemelle
-
-=item 1Chess Adventurer -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Alfonso-X -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Alpha -- Eric Bentzen -- bentzen1
-
-=item 1Chess Berlin -- Eric Bentzen -- bentzen2
-
-=item 1Chess Condal -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Harlequin -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Kingdom -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Leipzig -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Line -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Lucena -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Magnetic -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Mark -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Marroquin -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Maya -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Mediaeval -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Mrida -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Millennia -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Miscel -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Montreal -- Gary Katch -- katch
-
-=item 1Chess Motif -- Armando H. Marroquin -- marroquin
-
-=item 1Chess Plain -- Alan Hickey -- hickey
-
-=item 1Chess Regular -- Alistair Scott -- scott1
-
-=item 1Chess Usual -- Armando H. Marroquin -- scott2
-
-=item 1Chess Utrecht -- Hans Bodlaender -- bodlaender
-
-=item 1Tilburg -- Eric Schiller and Bill Cone -- tilburg
-
-=item 1Traveller Standard v3 -- Alan Cowderoy -- cowderoy
-
-=back
-
-These are available at L<http://www.enpassant.dk/chess/fonteng.htm> along
-with a good deal of useful information on chess desktop publishing.
-
-=head3 Font Designers Supported
-
-=over
-
-=item Eric Bentzen
-
-=item Bill Cone
-
-=item Alan Cowderoy
-
-=item Alan Hickey
-
-=item Gary Katch
-
-=item Armondo H. Marroquin
-
-=item Eric Schiller
-
-=item Alastair Scott
-
-=item Steve Smith
-
-=item Piet Tutelaers
-
-=back
-
-=head3 Borders and Such Like
-
-Some fonts, for example those designed by Armondo H. Marroquin support a variety of border
-styles and decorations. The border may be single or double, with square corners or rounded,
-and with an algebraic legend. These effects are supported by the addition of the necessary
-parameters to the allowed parameter list. In particular:
-
-=over
-
-=item * Border, values can be either 'single' or 'double' (default is 'single')
-
-=item * Corner, values can be either 'square' or 'rounded' (default is 'square')
-
-=item * Legend, values can be either 'yes' or 'no' (default is 'no')
-
-=back
-
-Again, letter case is not particularly important, 'yes' works as well as 'Yes' etc.
-As for those fonts that don't support a particular feature, B<epdstr> will fail silently, that
-is, the parameter will be ignored and processing will continue as though no such request
-had been made.
-
 =head2 epdlist(I<movelist>)
 
 Returns an array of strings that represent the conversion of
@@ -1514,26 +1508,328 @@ or can easily be reconstructed. As to why the module is called EPD, well I figur
 wasn't one and it wasn't the other, that left it up to me to choose--besides, who would want
 a module named after a swamp?!
 
-=head2 epdgetboard(I<epd>)
+=head2 epdset(I<epd>)
 
-Provides access to the 'board' with the current epd postition set up. The returned values are:
+For those instances where the game in question does not begin
+with a complete move list, this function allows the user to
+set the starting position using a 'EPD' string as described
+elsewhere in the document.
+
+=head2 epdstr(I<epd>|I<board>,I<type> [I<border>,I<corner>,I<legend>])
+
+Returns an array of strings that represent a diagramatic conversion of the
+specified B<epd> string or board postion to the specified B<type>. Parameters are passed as
+a anonymous hash, i.e. epdstr(epd => $position,type => 'diagram') or similar.
+
+=head3 Types Supported
+
+The following types are understood by B<epdstr>:
 
 =over
 
-=item $w - boolean, white to move?
+=item 'diagram'
 
-=item $Kc - boolean, has white castled king side?
+A plain ASCII diagram with simple border showing rank and file. Typical output:
 
-=item $Qc - boolean, has white castled queen side?
+ 8  rnbqkb r
+ 7  ppp pppp
+ 6   - - n -
+ 5  - -P- - 
+ 4   - - - -
+ 3  - - - - 
+ 2  PPPP PPP
+ 1  RNBQKBNR
+    abcdefgh
 
-=item $kc - boolean, has black castled king side?
+=item 'text'
 
-=item $qc - boolean, has black castled queen side?
+A plain ASCII diagram. Typical output:
 
-=item %board - hash board, keys are algebraic square names, values are occupying piece.
+ rnbqkb r
+ ppp pppp
+  - - n -
+ - -P- - 
+  - - - -
+ - - - - 
+ PPPP PPP
+ RNBQKBNR
+
+=item 'palview'
+
+An array of HTML information that represents the tabular diagram information for PalView.
+Typical output:
+
+<IMG SRC="jpc/br.gif"><IMG SRC="jpc/bn.gif"><IMG SRC="jpc/bb.gif"><IMG SRC="jpc/bq.gif"><IMG SRC="jpc/bk.gif"><IMG SRC="jpc/bb.gif"><IMG SRC="jpc/bn.gif"><IMG SRC="jpc/br.gif"><BR>
+<IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><IMG SRC="jpc/bp.gif"><BR>
+<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
+<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
+<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><BR>
+<IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/wn.gif"><BR>
+<IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><IMG SRC="jpc/wp.gif"><BR>
+<IMG SRC="jpc/wr.gif"><IMG SRC="jpc/wn.gif"><IMG SRC="jpc/wb.gif"><IMG SRC="jpc/wq.gif"><IMG SRC="jpc/wk.gif"><IMG SRC="jpc/wb.gif"><IMG SRC="jpc/i.gif"><IMG SRC="jpc/wr.gif"><BR>
+
+=item 'latex'
+
+The necessary text fragment to 'set' the diagram in LaTeX using 
+any variation of Piet Tutelars original chess12.tar.Z package. As given, the LaTeX
+command 'diagram' is used. As an example here is the source to test.tex:
+
+ %%
+ %% test.tex -- example LaTeX file to demonstrate output from Chess::PGN::EPD
+ %%
+ \documentclass{article}
+ \usepackage{chess}
+ \usepackage{bdfchess}
+ \begin{document}
+ \newenvironment{diagram}{\begin{nochess}}{$$\showboardwithnotation$$\end{nochess}}
+ %%
+ %% fragment as produced by epdstr(epd => $position,type => 'latex');
+ %%
+ \begin{diagram}
+ \board
+ {rnbqkb r}
+ {ppp pppp}
+ { * * n *}
+ {* *P* * }
+ { * * * *}
+ {* * * * }
+ {PPPP PPP}
+ {RNBQKBNR}
+ \end{diagram}
+ %%
+ %% end of fragment
+ %%
+ \end{document}
+
+=item 'linares'
+
+Alpine Electronics' LinaresDiagram font. Mapping also works with both HastingsDiagram
+and ZurichDiagram fonts. Single or double border, With or without algebraic legend.
+
+=item 'linares1'
+
+Standard mapping, single border, squares offset.
+
+=item 'linares2'
+
+Standard mapping, thick single border.
+
+=item 'tilburg'
+
+A borderless font designed by Eric Schiller and Bill Cone.
+
+=item 'marroquin'
+
+This type refers to any font designed by Armando H. Marroquin,
+excepting his FigurineSymbol fonts. They having a different purpose,
+have a different mapping.
+
+=item 'leschemelle'
+
+The map for Chess Cases designed by Matthieu Leschemelle.
+
+=item 'bentzen1'
+
+The map for Chess Alpha designed by Eric Bentzen.
+
+=item 'bentzen2'
+
+The map for Chess Berlin designed by Eric Bentzen.
+
+=item 'hickey'
+
+The map for Chess Plain designed by Alan Hickey.
+
+=item 'scott1'
+
+The map for Chess Regular a port of Adobe Cheq ported to
+True Type by Alistair Scott.
+
+=item 'scott2'
+
+The map for Chess Usual a modification of Chess Regular
+by Armando H. Marroquin.
+
+=item 'bodlaender'
+
+The map for Chess Utrecht designed by Hans Bodlaender.
+
+=item 'cowderoy'
+
+The map for Traveller Standard v3  designed by Alan Cowderoy.
 
 =back
 
+Note that 'type' is not case sensative so that 'latex' and 'LaTeX' will both
+work equally well.
+
+=head3 Fonts Supported
+
+List with font name, font author, and type name:
+
+=over
+
+=item Chess Cases -- Matthieu Leschemelle -- leschemelle
+
+=item Chess Adventurer -- Armando H. Marroquin -- marroquin
+
+=item Chess Alfonso-X -- Armando H. Marroquin -- marroquin
+
+=item Chess Alpha -- Eric Bentzen -- bentzen1
+
+=item Chess Berlin -- Eric Bentzen -- bentzen2
+
+=item Chess Condal -- Armando H. Marroquin -- marroquin
+
+=item Chess Harlequin -- Armando H. Marroquin -- marroquin
+
+=item Chess Kingdom -- Armando H. Marroquin -- marroquin
+
+=item Chess Leipzig -- Armando H. Marroquin -- marroquin
+
+=item Chess Line -- Armando H. Marroquin -- marroquin
+
+=item Chess Lucena -- Armando H. Marroquin -- marroquin
+
+=item Chess Magnetic -- Armando H. Marroquin -- marroquin
+
+=item Chess Mark -- Armando H. Marroquin -- marroquin
+
+=item Chess Marroquin -- Armando H. Marroquin -- marroquin
+
+=item Chess Maya -- Armando H. Marroquin -- marroquin
+
+=item Chess Mediaeval -- Armando H. Marroquin -- marroquin
+
+=item Chess Mrida -- Armando H. Marroquin -- marroquin
+
+=item Chess Millennia -- Armando H. Marroquin -- marroquin
+
+=item Chess Miscel -- Armando H. Marroquin -- marroquin
+
+=item Chess Montreal -- Gary Katch -- katch
+
+=item Chess Motif -- Armando H. Marroquin -- marroquin
+
+=item Chess Plain -- Alan Hickey -- hickey
+
+=item Chess Regular -- Alistair Scott -- scott1
+
+=item Chess Usual -- Armando H. Marroquin -- scott2
+
+=item Chess Utrecht -- Hans Bodlaender -- bodlaender
+
+=item Tilburg -- Eric Schiller and Bill Cone -- tilburg
+
+=item Traveller Standard v3 -- Alan Cowderoy -- cowderoy
+
+=back
+
+These are available at L<http://www.enpassant.dk/chess/fonteng.htm> along
+with a good deal of useful information on chess desktop publishing.
+
+=head3 Font Designers Supported
+
+=over
+
+=item Eric Bentzen
+
+=item Bill Cone
+
+=item Alan Cowderoy
+
+=item Alan Hickey
+
+=item Gary Katch
+
+=item Armondo H. Marroquin
+
+=item Eric Schiller
+
+=item Alastair Scott
+
+=item Steve Smith
+
+=item Piet Tutelaers
+
+=back
+
+=head3 Borders and Such Like
+
+Some fonts, for example those designed by Armondo H. Marroquin support a variety of border
+styles and decorations. The border may be single or double, with square corners or rounded,
+and with an algebraic legend. These effects are supported by the addition of the necessary
+parameters to the allowed parameter list. In particular:
+
+=over
+
+=item * Border, values can be either 'single' or 'double' (default is 'single')
+
+=item * Corner, values can be either 'square' or 'rounded' (default is 'square')
+
+=item * Legend, values can be either 'yes' or 'no' (default is 'no')
+
+=back
+
+Again, letter case is not particularly important, 'yes' works as well as 'Yes' etc.
+As for those fonts that don't support a particular feature, B<epdstr> will fail silently, that
+is, the parameter will be ignored and processing will continue as though no such request
+had been made.
+
+=head2 epdTaxonomy(I<options>)
+
+At one point the following was required in order to properly 'tag' a PGN file with opening
+names and information:
+
+ if ($ARGV[0]) {
+     my $pgn = new Chess::PGN::Parse($ARGV[0]) or die "Can't open $ARGV[0]: $!\n";
+     while ($pgn->read_game()) {
+         my @epd;
+ 
+         $pgn->parse_game();
+         @epd = reverse epdlist( @{$pgn->moves()} );
+         print '[ECO,"',epdcode('ECO',\@epd),"\"]\n";
+         print '[NIC,"',epdcode('NIC',\@epd),"\"]\n";
+         print '[Opening,"',epdcode('Opening',\@epd),"\"]\n";
+     }
+ }
+
+Not all that bad, but not all that clear either. As can be seen from the examples shown at the
+begining of this documentation, I've created a new subroutine called epdTaxonomy that replaces
+all of the above with:
+
+ if ($ARGV[0]) {
+     my $pgn = new Chess::PGN::Parse($ARGV[0]) or die "Can't open $ARGV[0]: $!\n";
+     while ($pgn->read_game()) {
+         my @epd;
+ 
+         $pgn->parse_game();
+         my @moves = @{ $game{'GameMoves'} };
+         print join("\n",epdTaxonomy(moves => \@moves,all => 1,astags => 1)),"\n";
+     }
+ }
+
+Clearly a win for the parsimonious team! This sub takes a single parameter, a hash with the following
+possibilities:
+
+=over
+
+=item 'moves' -- required in order to have something to work with.
+
+=item 'all' -- if true, create all three tags supported.
+
+=item 'astags' -- if true, create complete PGN header tags for the specified codes.
+
+=item 'eco' -- create tag information for ECO codes.
+
+=item 'nic' -- create tag information for NIC codes.
+
+=item 'opening' -- create tag information for Opening codes.
+
+=back
+
+See the description of epdcode for brief descriptions of what these codes represent.
+
 =head2 psquares(I<piece>,I<board>)
 
 Given a 'piece' (single character, uppercase for white, lowercase for black, KQRBNPkqrbnp) and
@@ -1546,13 +1842,17 @@ piece.
 
 =item epdcode - given a list of EPD strings, return the requested code or 'unknown'.
 
+=item epdfromto - given a list of moves, return an array of hashes which contain move information.
+
+=item epdgetboard - given an EPD string setup board and related. Either way return board and related information.
+
+=item epdlist - given a list of moves, return a list of EPD strings.
+
 =item epdset - allows the user to specifiy an initial position.
 
 =item epdstr - given an EPD string or a board, convert it to the specified string representation.
 
-=item epdlist - given a list of moves, return a list of EPD strings.
-
-=item epdgetboard - given an EPD string, setup current board and return current information.
+=item epdTaxonomy - one stop shopping for conversion of epd array to ECO, NIC and opening tag information.
 
 =item psquares - given the piece and the board, generate and return a list of squares occupied by that type of piece.
 
@@ -1562,11 +1862,19 @@ piece.
 
 =over
 
-=item DB_File
+=item Chess::PGN::Moves     => 0.04
 
-=item Chess::PGN::MOVES
+=item Chess::PGN::Parse     => 0.19
 
-=item Chess::PGN::Parse (useless without, not actually required though...)
+=item Test::More            => 0.94
+
+=item Text::CSV             => 1.18
+
+=item Storable              => 2.21
+
+=item Cwd                   => 3.31
+
+=item File::Spec::Functions => 3.3
 
 =back
 
